/*
 * @Descripttion: 图层定位
 * @version: 
 * @Author: 牛夏蕊
 * @Date: 2022-07-30 14:20:38
 * @LastEditors: 牛夏蕊
 * @LastEditTime: 2022-08-10 16:37:59
 */
/**
 * 根据行政区代码定位
 * @param XZQDM 行政区代码
 */
import * as Cesium from "cesium";
import mapRequest from "@/utils/map/mapRequest";
import { getErdsxqInfo } from "@/api/main/index.js";
import { addProjectionTransform } from "/src/utils/tools/CoordTransformate/CoordTransformate"
/**
 * @Desc: 飞行
 * @param {*} viewer
 * @param {*} destination
 * @param {*} lat
 * @param {*} height
 * @param {*} orientation
 * @param {*} pitch
 * @param {*} roll
 * @return {*}
 */
export function g_flyto(viewer, destination = { lon: 108.9670415016, lat: 39.2470366195, height: 50000 }, orientation = { heading: Cesium.Math.toRadians(0.0), pitch: Cesium.Math.toRadians(-30), roll: 0.0 }) {
    viewer.camera.flyTo({
        destination: Cesium.Cartesian3.fromDegrees(destination.lon, destination.lat, destination.height),
        orientation: {
            heading: orientation.heading,
            pitch: orientation.pitch,
            roll: orientation.roll
        }
    })
}
/**
 * 根据行政区代码定位
 * @param XZQDM 行政区代码
 */
export function globleXZQPosition(XZQDM, viewer) {
    getXZQExtents(XZQDM, function (extent) {
        extent && addXZQFeatures(extent, viewer);
    });
}
/**
 * 根据经纬度定位
 * @param x 经度
 * @param y 纬度
 * @param z 高程
 */
export function globleCoordinatePosition(x, y, z, name, viewer) {
    var x = parseFloat(x);
    var y = parseFloat(y);

    var cartesian3 = Cesium.Cartesian3.fromDegrees(x, y, 0)
    var pinBuilder = new Cesium.PinBuilder();
    var plotentity = viewer.entities.add({
        position: cartesian3,
        billboard: {
            image: pinBuilder.fromText(name, Cesium.Color.BLUE.withAlpha(0.75), 48).toDataURL(),
            distanceDisplayCondition: new Cesium.DistanceDisplayCondition(2000),
            verticalOrigin: Cesium.VerticalOrigin.BOTTOM
        },
        point: {
            heightReference: Cesium.HeightReference.CLAMP_TO_GROUND,
            pixelSize: 5,
            color: Cesium.Color.RED.withAlpha(1)
        }
    });
    g_flyto(viewer, { lon: x, lat: y, height: (z ? z : 5000) }, { heading: Cesium.Math.toRadians(0.0), pitch: Cesium.Math.toRadians(-90), roll: 0.0 })
}

//根据行政区代码XZQDM在服务中查询服务范围
export function getXZQExtents(XZQDM, callback) {
    getErdsxqInfo({}).then((res) => {
        res = typeof res == "string" ? JSON.parse(res) : res;
        var thisResult = null;
        var ip = res.data.xzqService + "/0/query";
        var sql = res.data.xzqField + "='" + XZQDM + "'"
        mapRequest({
            url: ip,
            method: 'get',
            params: {
                f: "json",
                where: sql,
                outSR: 3857
            }
        }).then((result) => {
            result = typeof result == "string" ? JSON.parse(result) : result;
            if (result.features.length > 0) {
                result = handle2Geojson(result, "arcgis");
                thisResult = result.features;
            }
            callback(thisResult)
        })
    });

    // return thisResult;
}
//添加行政区图层并进行定位
export function addXZQFeatures(features, viewer) {
    var inSR = "EPSG:" + features[0].arcgis.spatialReference.latestWkid; //3857
    var outSR = "EPSG:4326"
    viewer.entities.removeAll();
    var entity = null;
    for (var m = 0; m < features.length; m++) {
        var geom = features[m].geometry;
        var geomtype = geom.type.toLowerCase();
        if (geomtype.indexOf("polygon") > -1) {
            var points = [];
            addProjectionTransform(geom.coordinates[0][0], inSR, outSR, true, false, function (e) {
                let coordinates = e.result
                for (var i = 0; i < coordinates.length; i++) {
                    var lon = coordinates[i][0];
                    var lat = coordinates[i][1];
                    points.push(lon);
                    points.push(lat);
                }
                entity = viewer.entities.add({
                    polyline: {
                        clampToGround: true,
                        material: Cesium.Color.fromCssColorString("#ff0000"),
                        positions: Cesium.Cartesian3.fromDegreesArray(points),
                        width: 5
                    },
                    polygon: {
                        hierarchy: Cesium.Cartesian3.fromDegreesArray(points),
                        material: Cesium.Color.WHITE.withAlpha(0.15),
                        outline: false,
                        outlineColor: Cesium.Color.BLACK.withAlpha(0)
                    }
                });

                if (entity) {
                    let flyPromise = viewer.flyTo(entity, {
                        offset: {
                            heading: Cesium.Math.toRadians(0.0),
                            pitch: Cesium.Math.toRadians(-45),
                            range: 0
                        }
                    });
                    flyPromise.then(function (flyPromise) {
                        setTimeout(function () {
                            viewer.entities.remove(entity);
                            setTimeout(function () {
                                viewer.entities.add(entity);
                            }, 10);
                        }, 50);
                    });
                }
            })

        }
    }

}

//将行政区要素格式转换为geoJSON格式
export function handle2Geojson(data, type) {
    var geojson = {
        "type": "FeatureCollection",
        "features": []
    };
    type = type ? type : "arcgis";
    switch (type) {
        case "arcgis":
            var param = data;
            var arcgis = {
                displayFieldName: "",
                fieldAliases: {},
                fields: [],
                features: [param],
                geometryType: param.geometryType,
                spatialReference: param.spatialReference
            }
            if (param.geometryType == "esriGeometryPolygon" || param.geometry.rings) {
                geojson.features.push({
                    "type": "Feature",
                    "geometry": {
                        "type": "Polygon",
                        //"coordinates": [param.geometry.rings]
                        "coordinates": [param.features[0].geometry.rings]
                    },
                    "properties": param.features[0].attributes,
                    "arcgis": arcgis
                });
            }
            break;
        default:
            break;
    }
    return geojson;
}